@@ -15,15 +15,20 @@
 
 
 from betterproto import Message
+from merlin.schema.io import proto_utils
 
-from .registry import Registry, RegistryMixin
 from .schema import schema
-from .schema.schema import ColumnSchema, Schema
-from .schema.tag import Tag
-from .utils import proto_utils
+from .schema.schema import ColumnSchema, Schema, categorical_cardinalities
 
 # Other monkey-patching
 Message.HasField = proto_utils.has_field  # type: ignore
 Message.copy = proto_utils.copy_better_proto_message  # type: ignore
 
-__all__ = ["ColumnSchema", "Schema", "schema", "Tag", "Registry", "RegistryMixin"]
+__all__ = [
+    "ColumnSchema",
+    "Schema",
+    "schema",
+    "Registry",
+    "RegistryMixin",
+    "categorical_cardinalities",
+]
