@@ -18,15 +18,18 @@ from functools import partial
 from typing import Any, Callable, Dict, List, Optional, Text, Union
 
 import torch
+from merlin.models.utils.doc_utils import docstring_parameter
+from merlin.schema import Tags, TagsType
 
-from merlin_standard_lib import Schema, Tag
-from merlin_standard_lib.utils.doc_utils import docstring_parameter
+from merlin_standard_lib import Schema, categorical_cardinalities
 from merlin_standard_lib.utils.embedding_utils import get_embedding_sizes_from_schema
 
+from ..block.base import SequentialBlock
 from ..tabular.base import (
     TABULAR_MODULE_PARAMS_DOCSTRING,
     FilterFeatures,
     TabularAggregationType,
+    TabularTransformation,
     TabularTransformationType,
 )
 from ..utils.torch_utils import calculate_batch_size_from_input_size, get_output_sizes_from_schema
@@ -107,7 +110,7 @@ class EmbeddingFeatures(InputBlock):
         infer_embedding_sizes_multiplier: float = 2.0,
         embeddings_initializers: Optional[Dict[str, Callable[[Any], None]]] = None,
         combiner: str = "mean",
-        tags: Optional[Union[Tag, list, str]] = None,
+        tags: Optional[TagsType] = None,
         item_id: Optional[str] = None,
         automatic_build: bool = True,
         max_sequence_length: Optional[int] = None,
@@ -158,8 +161,14 @@ class EmbeddingFeatures(InputBlock):
         if tags:
             schema = schema.select_by_tag(tags)
 
-        if not item_id and schema.select_by_tag(["item_id"]).column_names:
-            item_id = schema.select_by_tag(["item_id"]).column_names[0]
+        _item_id = schema.select_by_tag(Tags.ITEM_ID)
+        if not item_id and len(_item_id) > 0:
+            if len(_item_id) > 1:
+                raise ValueError(
+                    "Multiple columns with tag ITEM_ID found. "
+                    "Please specify the item_id column name."
+                )
+            item_id = list(_item_id)[0].name
 
         embedding_dims = embedding_dims or {}
 
@@ -176,7 +185,7 @@ class EmbeddingFeatures(InputBlock):
         embeddings_initializers = embeddings_initializers or {}
 
         emb_config = {}
-        cardinalities = schema.categorical_cardinalities()
+        cardinalities = categorical_cardinalities(schema)
         for key, cardinality in cardinalities.items():
             embedding_size = embedding_dims.get(key, embedding_dim_default)
             embedding_initializer = embeddings_initializers.get(key, None)
@@ -249,6 +258,14 @@ class EmbeddingFeatures(InputBlock):
 
 
 class EmbeddingBagWrapper(torch.nn.EmbeddingBag):
+    """
+    Wrapper class for the PyTorch EmbeddingBag module.
+
+    This class extends the torch.nn.EmbeddingBag class and overrides
+    the forward method to handle 1D tensor inputs
+    by reshaping them to 2D as required by the EmbeddingBag.
+    """
+
     def forward(self, input, **kwargs):
         # EmbeddingBag requires 2D tensors (or offsets)
         if len(input.shape) == 1:
@@ -304,7 +321,7 @@ class SoftEmbeddingFeatures(EmbeddingFeatures):
         embeddings_initializers: Optional[Dict[str, Callable[[Any], None]]] = None,
         layer_norm: bool = True,
         combiner: str = "mean",
-        tags: Optional[Union[Tag, list, str]] = None,
+        tags: Optional[TagsType] = None,
         automatic_build: bool = True,
         max_sequence_length: Optional[int] = None,
         **kwargs,
@@ -353,7 +370,7 @@ class SoftEmbeddingFeatures(EmbeddingFeatures):
         embeddings_initializers = embeddings_initializers or {}
 
         sizes = {}
-        cardinalities = schema.categorical_cardinalities()
+        cardinalities = categorical_cardinalities(schema)
         for col_name in schema.column_names:
             # If this is NOT a categorical feature
             if col_name not in cardinalities:
@@ -397,6 +414,29 @@ class SoftEmbeddingFeatures(EmbeddingFeatures):
 
 
 class TableConfig:
+    """
+    Class to configure the embeddings lookup table for a categorical feature.
+
+    Attributes
+    ----------
+    vocabulary_size : int
+        The size of the vocabulary,
+        i.e., the cardinality of the categorical feature.
+    dim : int
+        The dimensionality of the embedding vectors.
+    initializer : Optional[Callable[[torch.Tensor], None]]
+        The initializer function for the embedding weights.
+        If None, the weights are initialized using a normal
+        distribution with mean 0.0 and standard deviation 0.05.
+    combiner : Optional[str]
+        The combiner operation used to aggregate bag of embeddings.
+        Possible options are "mean", "sum", and "sqrtn".
+        By default "mean".
+    name : Optional[str]
+        The name of the lookup table.
+        By default None.
+    """
+
     def __init__(
         self,
         vocabulary_size: int,
@@ -441,6 +481,23 @@ class TableConfig:
 
 
 class FeatureConfig:
+    """
+    Class to set the embeddings table of a categorical feature
+    with a maximum sequence length.
+
+    Attributes
+    ----------
+    table : TableConfig
+        Configuration for the lookup table,
+        which is used for embedding lookup and aggregation.
+    max_sequence_length : int, optional
+        Maximum sequence length for sequence features.
+        By default 0.
+    name : str, optional
+        The feature name.
+        By default None
+    """
+
     def __init__(
         self, table: TableConfig, max_sequence_length: int = 0, name: Optional[Text] = None
     ):
@@ -534,3 +591,147 @@ class PretrainedEmbeddingsInitializer(torch.nn.Module):
         with torch.no_grad():
             x.copy_(self.weight_matrix)
         x.requires_grad = self.trainable
+
+
+@docstring_parameter(
+    tabular_module_parameters=TABULAR_MODULE_PARAMS_DOCSTRING,
+)
+class PretrainedEmbeddingFeatures(InputBlock):
+    """Input block for pre-trained embeddings features.
+
+    For 3-D features, if sequence_combiner is set, the features are aggregated
+    using the second dimension (sequence length)
+
+    Parameters
+    ----------
+    features: List[str]
+        A list of the pre-trained embeddings feature names.
+        You typically will pass schema.select_by_tag(Tags.EMBEDDING).column_names,
+        as that is the tag added to pre-trained embedding features when using the
+        merlin.dataloader.ops.embeddings.EmbeddingOperator
+    pretrained_output_dims: Optional[Union[int, Dict[str, int]]]
+        If provided, it projects features to specified dim(s).
+        If an int, all features are projected to that dim.
+        If a dict, only features provided in the dict will be mapped to the specified dim,
+    sequence_combiner: Optional[Union[str, torch.nn.Module]], optional
+       A string ("mean", "sum", "max", "min") or torch.nn.Module specifying
+       how to combine the second dimension of the pre-trained embeddings if it is 3D.
+       Default is None (no sequence combiner used)
+    normalizer: Optional[Union[str, TabularTransformationType]]
+       A tabular layer (e.g.tr.TabularLayerNorm()) or string ("layer-norm") to be applied
+       to pre-trained embeddings after projected and sequence combined
+       Default is None (no normalization)
+    schema (Optional[Schema]): the schema of the input data.
+    {tabular_module_parameters}
+    """
+
+    def __init__(
+        self,
+        features: List[str],
+        pretrained_output_dims: Optional[Union[int, Dict[str, int]]] = None,
+        sequence_combiner: Optional[Union[str, torch.nn.Module]] = None,
+        pre: Optional[TabularTransformationType] = None,
+        post: Optional[TabularTransformationType] = None,
+        aggregation: Optional[TabularAggregationType] = None,
+        normalizer: Optional[TabularTransformationType] = None,
+        schema: Optional[Schema] = None,
+    ):
+        if isinstance(normalizer, str):
+            normalizer = TabularTransformation.parse(normalizer)
+        if not post:
+            post = normalizer
+        elif normalizer:
+            post = SequentialBlock(normalizer, post)  # type: ignore
+
+        super().__init__(pre=pre, post=post, aggregation=aggregation, schema=schema)
+        self.features = features
+        self.filter_features = FilterFeatures(features)
+        self.pretrained_output_dims = pretrained_output_dims
+        self.sequence_combiner = self.parse_combiner(sequence_combiner)
+
+    def build(self, input_size, **kwargs):
+        if input_size is not None:
+            if self.pretrained_output_dims:
+                self.projection = torch.nn.ModuleDict()
+                if isinstance(self.pretrained_output_dims, int):
+                    for key in self.features:
+                        self.projection[key] = torch.nn.Linear(
+                            input_size[key][-1], self.pretrained_output_dims
+                        )
+
+                elif isinstance(self.pretrained_output_dims, dict):
+                    for key in self.features:
+                        self.projection[key] = torch.nn.Linear(
+                            input_size[key][-1], self.pretrained_output_dims[key]
+                        )
+
+        return super().build(input_size, **kwargs)
+
+    @classmethod
+    def from_schema(
+        cls,
+        schema: Schema,
+        tags: Optional[TagsType] = None,
+        pretrained_output_dims=None,
+        sequence_combiner=None,
+        normalizer: Optional[Union[str, TabularTransformationType]] = None,
+        pre: Optional[TabularTransformationType] = None,
+        post: Optional[TabularTransformationType] = None,
+        aggregation: Optional[TabularAggregationType] = None,
+        **kwargs,
+    ):  # type: ignore
+        if tags:
+            schema = schema.select_by_tag(tags)
+
+        features = schema.column_names
+        return cls(
+            features=features,
+            pretrained_output_dims=pretrained_output_dims,
+            sequence_combiner=sequence_combiner,
+            pre=pre,
+            post=post,
+            aggregation=aggregation,
+            normalizer=normalizer,
+        )
+
+    def forward(self, inputs):
+        output = self.filter_features(inputs)
+        if self.pretrained_output_dims:
+            output = {key: self.projection[key](val) for key, val in output.items()}
+        if self.sequence_combiner:
+            for key, val in output.items():
+                if val.dim() > 2:
+                    output[key] = self.sequence_combiner(val, axis=1)
+
+        return output
+
+    def forward_output_size(self, input_sizes):
+        sizes = self.filter_features.forward_output_size(input_sizes)
+        if self.pretrained_output_dims:
+            if isinstance(self.pretrained_output_dims, dict):
+                sizes.update(
+                    {
+                        key: torch.Size(list(sizes[key][:-1]) + [self.pretrained_output_dims[key]])
+                        for key in self.features
+                    }
+                )
+            else:
+                sizes.update(
+                    {
+                        key: torch.Size(list(sizes[key][:-1]) + [self.pretrained_output_dims])
+                        for key in self.features
+                    }
+                )
+        return sizes
+
+    def parse_combiner(self, combiner):
+        if isinstance(combiner, str):
+            if combiner == "sum":
+                combiner = torch.sum
+            elif combiner == "max":
+                combiner = torch.max
+            elif combiner == "min":
+                combiner = torch.min
+            elif combiner == "mean":
+                combiner = torch.mean
+        return combiner
