@@ -17,10 +17,10 @@
 from typing import Dict, List, Optional, Tuple, Union
 
 import torch
+from merlin.models.utils.doc_utils import docstring_parameter
+from merlin.schema import Tags, TagsType
 
-from merlin_standard_lib import Schema, Tag
-from merlin_standard_lib.schema.tag import TagsType
-from merlin_standard_lib.utils.doc_utils import docstring_parameter
+from merlin_standard_lib import Schema
 
 from ..block.base import BlockOrModule, BuildableBlock, SequentialBlock
 from ..block.mlp import MLPBlock
@@ -115,7 +115,7 @@ class TabularSequenceFeatures(TabularFeatures):
         self,
         continuous_module: Optional[TabularModule] = None,
         categorical_module: Optional[TabularModule] = None,
-        text_embedding_module: Optional[TabularModule] = None,
+        pretrained_embedding_module: Optional[TabularModule] = None,
         projection_module: Optional[BlockOrModule] = None,
         masking: Optional[MaskSequence] = None,
         pre: Optional[TabularTransformationType] = None,
@@ -127,7 +127,7 @@ class TabularSequenceFeatures(TabularFeatures):
         super().__init__(
             continuous_module,
             categorical_module,
-            text_embedding_module,
+            pretrained_embedding_module,
             pre=pre,
             post=post,
             aggregation=aggregation,
@@ -141,8 +141,9 @@ class TabularSequenceFeatures(TabularFeatures):
     def from_schema(  # type: ignore
         cls,
         schema: Schema,
-        continuous_tags: Optional[Union[TagsType, Tuple[Tag]]] = (Tag.CONTINUOUS,),
-        categorical_tags: Optional[Union[TagsType, Tuple[Tag]]] = (Tag.CATEGORICAL,),
+        continuous_tags: Optional[Union[TagsType, Tuple[Tags]]] = (Tags.CONTINUOUS,),
+        categorical_tags: Optional[Union[TagsType, Tuple[Tags]]] = (Tags.CATEGORICAL,),
+        pretrained_embeddings_tags: Optional[Union[TagsType, Tuple[Tags]]] = (Tags.EMBEDDING,),
         aggregation: Optional[str] = None,
         automatic_build: bool = True,
         max_sequence_length: Optional[int] = None,
@@ -159,10 +160,10 @@ class TabularSequenceFeatures(TabularFeatures):
         ----------
         schema : DatasetSchema
             Dataset schema
-        continuous_tags : Optional[Union[DefaultTags, list, str]], optional
-            Tags to filter the continuous features, by default Tag.CONTINUOUS
-        categorical_tags : Optional[Union[DefaultTags, list, str]], optional
-            Tags to filter the categorical features, by default Tag.CATEGORICAL
+        continuous_tags : Optional[Union[TagsType, Tuple[Tags]]], optional
+            Tags to filter the continuous features, by default Tags.CONTINUOUS
+        categorical_tags : Optional[Union[TagsType, Tuple[Tags]]], optional
+            Tags to filter the categorical features, by default Tags.CATEGORICAL
         aggregation : Optional[str], optional
             Feature aggregation option, by default None
         automatic_build : bool, optional
@@ -194,6 +195,7 @@ class TabularSequenceFeatures(TabularFeatures):
             schema=schema,
             continuous_tags=continuous_tags,
             categorical_tags=categorical_tags,
+            pretrained_embeddings_tags=pretrained_embeddings_tags,
             aggregation=aggregation,
             automatic_build=automatic_build,
             max_sequence_length=max_sequence_length,
