@@ -16,9 +16,10 @@
 
 from typing import List, Optional, Tuple, Type, Union
 
-from merlin_standard_lib import Schema, Tag
-from merlin_standard_lib.schema.tag import TagsType
-from merlin_standard_lib.utils.doc_utils import docstring_parameter
+from merlin.models.utils.doc_utils import docstring_parameter
+from merlin.schema import Tags, TagsType
+
+from merlin_standard_lib import Schema
 
 from ..block.base import SequentialBlock
 from ..block.mlp import MLPBlock
@@ -32,7 +33,7 @@ from ..tabular.base import (
 )
 from ..utils.torch_utils import get_output_sizes_from_schema
 from .continuous import ContinuousFeatures
-from .embedding import EmbeddingFeatures, SoftEmbeddingFeatures
+from .embedding import EmbeddingFeatures, PretrainedEmbeddingFeatures, SoftEmbeddingFeatures
 
 TABULAR_FEATURES_PARAMS_DOCSTRING = """
     continuous_module: TabularModule, optional
@@ -60,12 +61,13 @@ class TabularFeatures(MergeTabular):
     CONTINUOUS_MODULE_CLASS: Type[TabularModule] = ContinuousFeatures
     EMBEDDING_MODULE_CLASS: Type[TabularModule] = EmbeddingFeatures
     SOFT_EMBEDDING_MODULE_CLASS: Type[TabularModule] = SoftEmbeddingFeatures
+    PRETRAINED_EMBEDDING_MODULE_CLASS: Type[TabularModule] = PretrainedEmbeddingFeatures
 
     def __init__(
         self,
         continuous_module: Optional[TabularModule] = None,
         categorical_module: Optional[TabularModule] = None,
-        text_embedding_module: Optional[TabularModule] = None,
+        pretrained_embedding_module: Optional[TabularModule] = None,
         pre: Optional[TabularTransformationType] = None,
         post: Optional[TabularTransformationType] = None,
         aggregation: Optional[TabularAggregationType] = None,
@@ -77,8 +79,8 @@ class TabularFeatures(MergeTabular):
             to_merge["continuous_module"] = continuous_module
         if categorical_module:
             to_merge["categorical_module"] = categorical_module
-        if text_embedding_module:
-            to_merge["text_embedding_module"] = text_embedding_module
+        if pretrained_embedding_module:
+            to_merge["pretrained_embedding_module"] = pretrained_embedding_module
 
         assert to_merge != {}, "Please provide at least one input layer"
         super(TabularFeatures, self).__init__(
@@ -118,8 +120,9 @@ class TabularFeatures(MergeTabular):
     def from_schema(  # type: ignore
         cls,
         schema: Schema,
-        continuous_tags: Optional[Union[TagsType, Tuple[Tag]]] = (Tag.CONTINUOUS,),
-        categorical_tags: Optional[Union[TagsType, Tuple[Tag]]] = (Tag.CATEGORICAL,),
+        continuous_tags: Optional[Union[TagsType, Tuple[Tags]]] = (Tags.CONTINUOUS,),
+        categorical_tags: Optional[Union[TagsType, Tuple[Tags]]] = (Tags.CATEGORICAL,),
+        pretrained_embeddings_tags: Optional[Union[TagsType, Tuple[Tags]]] = (Tags.EMBEDDING,),
         aggregation: Optional[str] = None,
         automatic_build: bool = True,
         max_sequence_length: Optional[int] = None,
@@ -133,10 +136,10 @@ class TabularFeatures(MergeTabular):
         ----------
         schema : DatasetSchema
             Dataset schema
-        continuous_tags : Optional[Union[DefaultTags, list, str]], optional
-            Tags to filter the continuous features, by default Tag.CONTINUOUS
-        categorical_tags : Optional[Union[DefaultTags, list, str]], optional
-            Tags to filter the categorical features, by default Tag.CATEGORICAL
+        continuous_tags : Optional[Union[TagsType, Tuple[Tags]]], optional
+            Tags to filter the continuous features, by default Tags.CONTINUOUS
+        categorical_tags : Optional[Union[TagsType, Tuple[Tags]]], optional
+            Tags to filter the categorical features, by default Tags.CATEGORICAL
         aggregation : Optional[str], optional
             Feature aggregation option, by default None
         automatic_build : bool, optional
@@ -155,7 +158,12 @@ class TabularFeatures(MergeTabular):
         TabularFeatures
             Returns ``TabularFeatures`` from a dataset schema
         """
-        maybe_continuous_module, maybe_categorical_module = None, None
+        maybe_continuous_module, maybe_categorical_module, maybe_pretrained_module = (
+            None,
+            None,
+            None,
+        )
+
         if continuous_tags:
             if continuous_soft_embeddings:
                 maybe_continuous_module = cls.SOFT_EMBEDDING_MODULE_CLASS.from_schema(
@@ -171,11 +179,15 @@ class TabularFeatures(MergeTabular):
             maybe_categorical_module = cls.EMBEDDING_MODULE_CLASS.from_schema(
                 schema, tags=categorical_tags, **kwargs
             )
+        if pretrained_embeddings_tags:
+            maybe_pretrained_module = cls.PRETRAINED_EMBEDDING_MODULE_CLASS.from_schema(
+                schema, tags=pretrained_embeddings_tags, **kwargs
+            )
 
         output = cls(
             continuous_module=maybe_continuous_module,
             categorical_module=maybe_categorical_module,
-            text_embedding_module=None,
+            pretrained_embedding_module=maybe_pretrained_module,
             aggregation=aggregation,
         )
 
@@ -218,3 +230,10 @@ class TabularFeatures(MergeTabular):
             return self.to_merge["categorical_module"]
 
         return None
+
+    @property
+    def pretrained_module(self) -> Optional[TabularModule]:
+        if "pretrained_embedding_module" in self.to_merge:
+            return self.to_merge["pretrained_embedding_module"]
+
+        return None
