@@ -24,18 +24,20 @@ from typing import Callable, Dict, Iterable, List, Optional, Type, Union, cast
 import numpy as np
 import torch
 import torchmetrics as tm
+from merlin.models.utils.registry import camelcase_to_snakecase
 from merlin.schema import ColumnSchema
 from merlin.schema import Schema as Core_Schema
+from merlin.schema import Tags
 from tqdm import tqdm
 from transformers.modeling_utils import SequenceSummary
 
-from merlin_standard_lib import Schema, Tag
-from merlin_standard_lib.registry import camelcase_to_snakecase
+from merlin_standard_lib import Schema
 
 from ..block.base import BlockBase, BlockOrModule, BlockType
 from ..features.base import InputBlock
 from ..features.sequence import TabularFeaturesType
-from ..typing import TabularData, TensorOrTabularData
+from ..typing import TabularData
+from ..utils.padding import pad_inputs
 from ..utils.torch_utils import LossMixin, MetricsMixin
 
 
@@ -45,7 +47,6 @@ def name_fn(name, inp):
 
 class PredictionTask(torch.nn.Module, LossMixin, MetricsMixin):
     """Individual prediction-task of a model.
-
     Parameters
     ----------
     loss: torch.nn.Module
@@ -108,7 +109,6 @@ class PredictionTask(torch.nn.Module, LossMixin, MetricsMixin):
         """
         The method will be called when block is converted to a model,
         i.e when linked to prediction head.
-
         Parameters
         ----------
         block:
@@ -166,6 +166,13 @@ class PredictionTask(torch.nn.Module, LossMixin, MetricsMixin):
         if training or testing:
             # add support of computing the loss inside the forward
             # and return a dictionary as standard output
+            if self.summary_type is None:
+                if targets.dim() != 2:
+                    raise ValueError(
+                        "If `summary_type==None`, targets are expected to be a 2D tensor, "
+                        f"but got a tensor with shape {targets.shape}"
+                    )
+
             loss = self.loss(x, target=targets)
             return {"loss": loss, "labels": targets, "predictions": x}
 
@@ -191,7 +198,6 @@ class PredictionTask(torch.nn.Module, LossMixin, MetricsMixin):
         predictions: torch.Tensor,
         targets: torch.Tensor,
     ) -> Dict[str, torch.Tensor]:
-
         outputs = {}
 
         predictions = self.forward_to_prediction_fn(cast(torch.Tensor, predictions))
@@ -224,7 +230,6 @@ class PredictionTask(torch.nn.Module, LossMixin, MetricsMixin):
 
 class Head(torch.nn.Module, LossMixin, MetricsMixin):
     """Head of a Model, a head has a single body but could have multiple prediction-tasks.
-
     Parameters
     ----------
     body: Block
@@ -269,7 +274,6 @@ class Head(torch.nn.Module, LossMixin, MetricsMixin):
 
     def build(self, inputs=None, device=None, task_blocks=None):
         """Build each prediction task that's part of the head.
-
         Parameters
         ----------
         body
@@ -306,7 +310,6 @@ class Head(torch.nn.Module, LossMixin, MetricsMixin):
         inputs: Optional[TabularFeaturesType] = None,
     ) -> "Head":
         """Instantiate a Head from a Schema through tagged targets.
-
         Parameters
         ----------
         schema: DatasetSchema
@@ -316,7 +319,6 @@ class Head(torch.nn.Module, LossMixin, MetricsMixin):
         task_weight_dict
         loss_reduction
         inputs
-
         Returns
         -------
         Head
@@ -327,11 +329,11 @@ class Head(torch.nn.Module, LossMixin, MetricsMixin):
 
         from .prediction_task import BinaryClassificationTask, RegressionTask
 
-        for binary_target in schema.select_by_tag(Tag.BINARY_CLASSIFICATION).column_names:
+        for binary_target in schema.select_by_tag([Tags.BINARY, Tags.CLASSIFICATION]).column_names:
             tasks.append(BinaryClassificationTask(binary_target))
             task_weights.append(task_weight_dict.get(binary_target, 1.0))
 
-        for regression_target in schema.select_by_tag(Tag.REGRESSION).column_names:
+        for regression_target in schema.select_by_tag(Tags.REGRESSION).column_names:
             tasks.append(RegressionTask(regression_target))
             task_weights.append(task_weight_dict.get(regression_target, 1.0))
 
@@ -348,12 +350,10 @@ class Head(torch.nn.Module, LossMixin, MetricsMixin):
 
     def pop_labels(self, inputs: TabularData) -> TabularData:
         """Pop the labels from the different prediction_tasks from the inputs.
-
         Parameters
         ----------
         inputs: TabularData
             Input dictionary containing all targets.
-
         Returns
         -------
         TabularData
@@ -371,10 +371,13 @@ class Head(torch.nn.Module, LossMixin, MetricsMixin):
         testing: bool = False,
         targets: Union[torch.Tensor, TabularData] = None,
         call_body: bool = False,
+        top_k: Optional[int] = None,
         **kwargs,
     ) -> Union[torch.Tensor, TabularData]:
         outputs = {}
 
+        from transformers4rec.torch.model.prediction_task import NextItemPredictionTask
+
         if call_body:
             body_outputs = self.body(body_outputs, training=training, testing=testing, **kwargs)
 
@@ -388,8 +391,7 @@ class Head(torch.nn.Module, LossMixin, MetricsMixin):
                 else:
                     label = targets
                 if label is not None:
-                    # Remove dimension `1` as merlin dataloader returns tensors of shape (-1, 1)
-                    label = torch.squeeze(label.float(), -1)
+                    label = label.float()
                 task_output = task(
                     body_outputs, targets=label, training=training, testing=testing, **kwargs
                 )
@@ -401,9 +403,20 @@ class Head(torch.nn.Module, LossMixin, MetricsMixin):
             outputs = {"loss": loss, "labels": labels, "predictions": predictions}
         else:
             for name, task in self.prediction_task_dict.items():
-                outputs[name] = task(
-                    body_outputs, targets=targets, training=training, testing=testing, **kwargs
-                )
+                if isinstance(task, NextItemPredictionTask):
+                    outputs[name] = task(
+                        body_outputs,
+                        targets=targets,
+                        training=training,
+                        testing=testing,
+                        top_k=top_k,
+                        **kwargs,
+                    )
+
+                else:
+                    outputs[name] = task(
+                        body_outputs, targets=targets, training=training, testing=testing, **kwargs
+                    )
 
         return outputs
 
@@ -413,7 +426,6 @@ class Head(torch.nn.Module, LossMixin, MetricsMixin):
         targets: Union[torch.Tensor, TabularData],
     ) -> Dict[str, Union[Dict[str, torch.Tensor], torch.Tensor]]:
         """Calculate metrics of the task(s) set in the Head instance.
-
         Parameters
         ----------
         predictions: Union[torch.Tensor, TabularData]
@@ -468,7 +480,6 @@ class Head(torch.nn.Module, LossMixin, MetricsMixin):
 
     def to_model(self, **kwargs) -> "Model":
         """Convert the head to a Model.
-
         Returns
         -------
         Model
@@ -485,9 +496,10 @@ class Model(torch.nn.Module, LossMixin, MetricsMixin):
         head_reduction: str = "mean",
         optimizer: Type[torch.optim.Optimizer] = torch.optim.Adam,
         name: str = None,
+        max_sequence_length: Optional[int] = None,
+        top_k: Optional[int] = None,
     ):
         """Model class that can aggregate one or multiple heads.
-
         Parameters
         ----------
         head: Head
@@ -500,6 +512,12 @@ class Model(torch.nn.Module, LossMixin, MetricsMixin):
             Optimizer-class to use during fitting
         name: str, optional
             Name of the model.
+        max_sequence_length: int, optional
+            The maximum sequence length supported by the model.
+            Used to truncate sequence inputs longer than this value.
+        top_k: int, optional
+            The number of items to return at the inference step once the model is deployed.
+            Default is None, which will return all items.
         """
         if head_weights:
             if not isinstance(head_weights, list):
@@ -516,17 +534,17 @@ class Model(torch.nn.Module, LossMixin, MetricsMixin):
         self.head_weights = head_weights or [1.0] * len(head)
         self.head_reduction = head_reduction
         self.optimizer = optimizer
+        self.max_sequence_length = max_sequence_length
+        self.top_k = top_k
 
-    def forward(
-        self, inputs: TensorOrTabularData, targets=None, training=False, testing=False, **kwargs
-    ):
+    def forward(self, inputs: TabularData, targets=None, training=False, testing=False, **kwargs):
         # Convert inputs to float32 which is the default type, expected by PyTorch
         for name, val in inputs.items():
             if torch.is_floating_point(val):
                 inputs[name] = val.to(torch.float32)
-        # Squeeze second dimension `1` of non-list inputs
-        for name, val in inputs.items():
-            inputs[name] = torch.squeeze(val, -1)
+
+        # pad ragged inputs
+        inputs = pad_inputs(inputs, self.max_sequence_length)
 
         if isinstance(targets, dict) and len(targets) == 0:
             # `pyarrow`` dataloader is returning {} instead of None
@@ -566,6 +584,7 @@ class Model(torch.nn.Module, LossMixin, MetricsMixin):
                         targets=targets,
                         training=training,
                         testing=testing,
+                        top_k=self.top_k,
                         **kwargs,
                     )
                 )
@@ -580,7 +599,6 @@ class Model(torch.nn.Module, LossMixin, MetricsMixin):
         targets: Union[torch.Tensor, TabularData],
     ) -> Dict[str, Union[Dict[str, torch.Tensor], torch.Tensor]]:
         """Calculate metrics of the task(s) set in the Head instance.
-
         Parameters
         ----------
         predictions: Union[torch.Tensor, TabularData]
@@ -729,6 +747,9 @@ class Model(torch.nn.Module, LossMixin, MetricsMixin):
         schemas = []
         for head in self.heads:
             schemas.append(head.body.inputs.schema)
+        if all(isinstance(s, Core_Schema) for s in schemas):
+            return sum(schemas, Core_Schema())
+
         model_schema = sum(schemas, Schema())
 
         # TODO: rework T4R to use Merlin Schemas.
@@ -739,31 +760,31 @@ class Model(torch.nn.Module, LossMixin, MetricsMixin):
 
             dtype = {0: np.float32, 2: np.int64, 3: np.float32}[column.type]
             tags = column.tags
-            is_list = column.value_count.max > 0
+            dims = None
+            if column.value_count.max > 0:
+                dims = (None, (column.value_count.min, column.value_count.max))
             int_domain = {"min": column.int_domain.min, "max": column.int_domain.max}
             properties = {
                 "int_domain": int_domain,
             }
 
             col_schema = ColumnSchema(
-                name,
-                dtype=dtype,
-                tags=tags,
-                properties=properties,
-                is_list=is_list,
-                is_ragged=False,
+                name, dtype=dtype, tags=tags, properties=properties, dims=dims
             )
             core_schema[name] = col_schema
         return core_schema
 
     @property
     def output_schema(self):
+        from merlin.schema import Tags
+
         from .prediction_task import BinaryClassificationTask, RegressionTask
 
         # if the model has one head with one task, the output is a tensor
         # if multiple heads and/or multiple prediction task, the output is a dictionary
         output_cols = []
         for head in self.heads:
+            dims = None
             for name, task in head.prediction_task_dict.items():
                 target_dim = task.target_dim
                 int_domain = {"min": target_dim, "max": target_dim}
@@ -771,22 +792,48 @@ class Model(torch.nn.Module, LossMixin, MetricsMixin):
                     isinstance(task, (BinaryClassificationTask, RegressionTask))
                     and not task.summary_type
                 ):
-                    is_list = True
+                    dims = (None, (1, None))
+                elif (
+                    isinstance(task, (BinaryClassificationTask, RegressionTask))
+                    and task.summary_type
+                ):
+                    dims = (None,)
                 else:
-                    is_list = False
+                    dims = (None, task.target_dim)
                 properties = {
                     "int_domain": int_domain,
                 }
-                col_schema = ColumnSchema(
-                    name, dtype=np.float32, properties=properties, is_list=is_list, is_ragged=False
-                )
-                output_cols.append(col_schema)
+                # in case one sets top_k at the inference step we return two outputs
+                if self.top_k:
+                    # be sure categ item-id dtype in model.input schema and output schema matches
+                    col_name = self.input_schema.select_by_tag(Tags.ITEM_ID).column_names[0]
+                    col_dtype = (
+                        self.input_schema.select_by_tag(Tags.ITEM_ID)
+                        .column_schemas[col_name]
+                        .dtype.name
+                    )
+                    col_schema_scores = ColumnSchema(
+                        "item_id_scores", dtype=np.float32, properties=properties, dims=dims
+                    )
+                    col_schema_ids = ColumnSchema(
+                        "item_ids", dtype=np.dtype(col_dtype), properties=properties, dims=dims
+                    )
+                    output_cols.append(col_schema_scores)
+                    output_cols.append(col_schema_ids)
+                else:
+                    col_schema = ColumnSchema(
+                        name, dtype=np.float32, properties=properties, dims=dims
+                    )
+                    output_cols.append(col_schema)
 
         return Core_Schema(output_cols)
 
+    @property
+    def prediction_tasks(self):
+        return [task for head in self.heads for task in list(head.prediction_task_dict.values())]
+
     def save(self, path: Union[str, os.PathLike], model_name="t4rec_model_class"):
         """Saves the model to f"{export_path}/{model_name}.pkl" using `cloudpickle`
-
         Parameters
         ----------
         path : Union[str, os.PathLike]
@@ -811,7 +858,6 @@ class Model(torch.nn.Module, LossMixin, MetricsMixin):
     @classmethod
     def load(cls, path: Union[str, os.PathLike], model_name="t4rec_model_class") -> "Model":
         """Loads a T4Rec model that was saved with `model.save()`.
-
         Parameters
         ----------
         path : Union[str, os.PathLike]
