@@ -41,6 +41,7 @@ from merlin_standard_lib import Schema
 
 from ..config.trainer import T4RecTrainingArguments
 from .model.base import Model
+from .model.prediction_task import NextItemPredictionTask
 from .utils.data_utils import T4RecDataLoader
 from .utils.torch_utils import nested_concat, nested_detach, nested_numpify, nested_truncate
 
@@ -100,7 +101,6 @@ class Trainer(BaseTrainer):
         incremental_logging: bool = False,
         **kwargs,
     ):
-
         mock_dataset = DatasetMock()
 
         self.incremental_logging = incremental_logging
@@ -128,6 +128,14 @@ class Trainer(BaseTrainer):
         self.schema = schema
         self.incremental_logging = incremental_logging
 
+        # Set global_rank and global_size if DDP is used
+        if self.args.local_rank != -1:
+            self.device = self.local_rank = self.args.local_rank
+            self.global_size = self.args.world_size
+        else:
+            self.device = self.local_rank = None
+            self.global_size = None
+
     def get_train_dataloader(self):
         """
         Set the train dataloader to use by Trainer.
@@ -140,14 +148,6 @@ class Trainer(BaseTrainer):
 
         assert self.schema is not None, "schema is required to generate Train Dataloader"
 
-        # Set global_rank and global_size if DDP is used
-        if self.args.local_rank != -1:
-            local_rank = self.args.local_rank
-            global_size = self.args.world_size
-        else:
-            local_rank = None
-            global_size = None
-
         return T4RecDataLoader.parse(self.args.data_loader_engine).from_schema(
             self.schema,
             self.train_dataset_or_path,
@@ -156,8 +156,9 @@ class Trainer(BaseTrainer):
             drop_last=self.args.dataloader_drop_last,
             shuffle=True,
             shuffle_buffer_size=self.args.shuffle_buffer_size,
-            global_rank=local_rank,
-            global_size=global_size,
+            global_rank=self.local_rank,
+            global_size=self.global_size,
+            device=self.device,
         )
 
     def get_eval_dataloader(self, eval_dataset=None):
@@ -174,6 +175,7 @@ class Trainer(BaseTrainer):
             raise ValueError("Trainer: evaluation requires an eval_dataset.")
         eval_dataset = eval_dataset if eval_dataset is not None else self.eval_dataset
         assert self.schema is not None, "schema is required to generate Eval Dataloader"
+
         return T4RecDataLoader.parse(self.args.data_loader_engine).from_schema(
             self.schema,
             self.eval_dataset_or_path,
@@ -182,6 +184,9 @@ class Trainer(BaseTrainer):
             drop_last=self.args.dataloader_drop_last,
             shuffle=False,
             shuffle_buffer_size=self.args.shuffle_buffer_size,
+            global_rank=self.local_rank,
+            global_size=self.global_size,
+            device=self.device,
         )
 
     def get_test_dataloader(self, test_dataset=None):
@@ -206,6 +211,9 @@ class Trainer(BaseTrainer):
             drop_last=self.args.dataloader_drop_last,
             shuffle=False,
             shuffle_buffer_size=self.args.shuffle_buffer_size,
+            global_rank=self.local_rank,
+            global_size=self.global_size,
+            device=self.device,
         )
 
     def num_examples(self, dataloader: DataLoader):
@@ -352,7 +360,7 @@ class Trainer(BaseTrainer):
         inputs = self._prepare_inputs(inputs)
         inputs, targets = inputs
         with torch.no_grad():
-            if self.use_amp:
+            if self._use_cuda_amp:
                 with autocast():
                     outputs = model(inputs, targets=targets, training=training, testing=testing)
             else:
@@ -379,6 +387,18 @@ class Trainer(BaseTrainer):
 
         return (loss, predictions, labels, other_outputs)
 
+    @property
+    def _use_cuda_amp(self):
+        """
+        Check for CUDA AMP that is compatible with versions of the
+        transformers package before and after version 4.20 (which
+        renamed the property `use_amp` to `use_cuda_amp`)
+        """
+        try:
+            return self.use_cuda_amp
+        except AttributeError:
+            return self.use_amp
+
     def evaluation_loop(
         self,
         dataloader: DataLoader,
@@ -438,7 +458,7 @@ class Trainer(BaseTrainer):
         logger.info("***** Running %s *****", description)
         logger.info("  Batch size = %d", batch_size)
 
-        preds_item_ids_scores_host: Union[torch.Tensor, List[torch.Tensor]] = None
+        preds_host: Union[torch.Tensor, List[torch.Tensor], Dict[str, torch.Tensor]] = None
         labels_host: Union[torch.Tensor, List[torch.Tensor]] = None
 
         if metric_key_prefix == "train" and self.args.eval_steps_on_train_set:
@@ -455,11 +475,11 @@ class Trainer(BaseTrainer):
         # Initialize containers
         # losses/preds/labels on GPU/TPU (accumulated for eval_accumulation_steps)
         losses_host = None
-        preds_item_ids_scores_host = None
+        preds_host = None
         labels_host = None
         # losses/preds/labels on CPU (final containers)
         all_losses = None
-        all_preds_item_ids_scores = None
+        all_preds = None
         all_labels = None
         # Will be useful when we have an iterable dataset so don't know its length.
         observed_num_examples = 0
@@ -508,37 +528,77 @@ class Trainer(BaseTrainer):
                     if labels_host is None
                     else nested_concat(labels_host, labels, padding_index=0)
                 )
-            if preds is not None and self.args.predict_top_k > 0:
+
+            if (
+                preds is not None
+                and any(isinstance(x, NextItemPredictionTask) for x in model.prediction_tasks)
+                and (self.args.predict_top_k or self.model.top_k)
+            ):
+                # get outputs of next-item scores
                 if isinstance(preds, dict):
-                    assert (
-                        "next-item" in preds
-                    ), "Top-k prediction is specific to NextItemPredictionTask"
-                    preds = preds["next-item"]
-                preds_sorted_item_scores, preds_sorted_item_ids = torch.topk(
-                    preds, k=self.args.predict_top_k, dim=-1
-                )
-                self._maybe_log_predictions(
-                    labels,
-                    preds_sorted_item_ids,
-                    preds_sorted_item_scores,
-                    # outputs["pred_metadata"],
-                    metrics_results_detailed,
-                    metric_key_prefix,
-                )
-                # The output predictions will be a tuple with the ranked top-n item ids,
-                # and item recommendation scores
-                preds_item_ids_scores = (
-                    preds_sorted_item_ids,
-                    preds_sorted_item_scores,
-                )
-                preds_item_ids_scores_host = (
-                    preds_item_ids_scores
-                    if preds_item_ids_scores_host is None
-                    else nested_concat(
-                        preds_item_ids_scores_host,
-                        preds_item_ids_scores,
+                    pred_next_item = preds["next-item"]
+                else:
+                    pred_next_item = preds
+
+                preds_sorted_item_scores = None
+                preds_sorted_item_ids = None
+
+                if self.model.top_k is not None and isinstance(pred_next_item, (list, tuple)):
+                    preds_sorted_item_scores, preds_sorted_item_ids = pred_next_item
+
+                    if self.args.predict_top_k:
+                        if self.args.predict_top_k > self.model.top_k:
+                            raise ValueError(
+                                "The args.predict_top_k should not be larger than model.top_k. "
+                                "The model.top_k is available to support inference (e.g. when "
+                                "serving with Triton Inference Server) to return only the top-k "
+                                "predicted items ids and their scores."
+                                "When doing offline predictions with `trainer.predict(), "
+                                "if you set model.top_k, the model will also limit the number of "
+                                "predictions output from trainer.predict(). "
+                                "In that case, you want either to reduce args.predict_top_k or "
+                                "increase model.top_k, so that args.predict_top_k is "
+                                "not larger than model.top_k."
+                            )
+                        preds_sorted_item_scores = preds_sorted_item_scores[
+                            :, : self.args.predict_top_k
+                        ]
+                        preds_sorted_item_ids = preds_sorted_item_ids[:, : self.args.predict_top_k]
+                elif self.args.predict_top_k:
+                    preds_sorted_item_scores, preds_sorted_item_ids = torch.topk(
+                        pred_next_item, k=self.args.predict_top_k, dim=-1
                     )
+
+                if preds_sorted_item_scores is not None:
+                    self._maybe_log_predictions(
+                        labels,
+                        preds_sorted_item_ids,
+                        preds_sorted_item_scores,
+                        # outputs["pred_metadata"],
+                        metrics_results_detailed,
+                        metric_key_prefix,
+                    )
+                    # The output predictions will be a tuple with the ranked top-n item ids,
+                    # and item recommendation scores
+                    if isinstance(preds, dict):
+                        preds["next-item"] = (
+                            preds_sorted_item_ids,
+                            preds_sorted_item_scores,
+                        )
+                    else:
+                        preds = (
+                            preds_sorted_item_ids,
+                            preds_sorted_item_scores,
+                        )
+
+            preds_host = (
+                preds
+                if preds_host is None
+                else nested_concat(
+                    preds_host,
+                    preds,
                 )
+            )
 
             self.control = self.callback_handler.on_prediction_step(
                 self.args, self.state, self.control
@@ -564,19 +624,19 @@ class Trainer(BaseTrainer):
                         if all_labels is None
                         else nested_concat(all_labels, labels, padding_index=0)
                     )
-                if preds_item_ids_scores_host is not None:
-                    preds_item_ids_scores = nested_numpify(preds_item_ids_scores_host)
-                    all_preds_item_ids_scores = (
-                        preds_item_ids_scores
-                        if all_preds_item_ids_scores is None
+                if preds_host is not None:
+                    preds = nested_numpify(preds_host)
+                    all_preds = (
+                        preds
+                        if all_preds is None
                         else nested_concat(
-                            all_preds_item_ids_scores,
-                            preds_item_ids_scores,
+                            all_preds,
+                            preds,
                         )
                     )
 
                 # Set back to None to begin a new accumulation
-                losses_host, preds_item_ids_scores_host, labels_host = None, None, None
+                losses_host, preds_host, labels_host = None, None, None
 
         if self.args.past_index and hasattr(self, "_past"):
             # Clean the state at the end of the evaluation loop
@@ -593,14 +653,14 @@ class Trainer(BaseTrainer):
             all_labels = (
                 labels if all_labels is None else nested_concat(all_labels, labels, padding_index=0)
             )
-        if preds_item_ids_scores_host is not None:
-            preds_item_ids_scores = nested_numpify(preds_item_ids_scores_host)
-            all_preds_item_ids_scores = (
-                preds_item_ids_scores
-                if all_preds_item_ids_scores is None
+        if preds_host is not None:
+            preds_host = nested_numpify(preds_host)
+            all_preds = (
+                preds_host
+                if all_preds is None
                 else nested_concat(
-                    all_preds_item_ids_scores,
-                    preds_item_ids_scores,
+                    all_preds,
+                    preds_host,
                 )
             )
         # Get Number of samples :
@@ -612,8 +672,8 @@ class Trainer(BaseTrainer):
         # samplers has been rounded to a multiple of batch_size, so we truncate.
         if all_losses is not None:
             all_losses = all_losses[:num_samples]
-        if all_preds_item_ids_scores is not None:
-            all_preds_item_ids_scores = nested_truncate(all_preds_item_ids_scores, num_samples)
+        if all_preds is not None:
+            all_preds = nested_truncate(all_preds, num_samples)
         if all_labels is not None:
             all_labels = nested_truncate(all_labels, num_samples)
 
@@ -632,7 +692,7 @@ class Trainer(BaseTrainer):
             metrics[f"{metric_key_prefix}_/loss"] = all_losses.mean().item()
 
         return EvalLoopOutput(
-            predictions=all_preds_item_ids_scores,
+            predictions=all_preds,
             label_ids=all_labels,
             metrics=metrics,
             num_samples=num_examples,
@@ -703,7 +763,7 @@ class Trainer(BaseTrainer):
         torch.random.set_rng_state(checkpoint_rng_state["cpu"])
         torch.cuda.random.set_rng_state_all(checkpoint_rng_state["cuda"])
         # Restoring AMP scaler
-        if self.use_amp:
+        if self._use_cuda_amp:
             self.scaler.load_state_dict(torch.load(os.path.join(checkpoint_path, "scaler.pt")))
 
     @property
