@@ -17,10 +17,11 @@
 import random
 from typing import Any, Dict, Optional
 
+import numpy as np
 import torch
+from merlin.schema.io.proto_utils import has_field
 
 from merlin_standard_lib import Schema
-from merlin_standard_lib.utils.proto_utils import has_field
 
 from ..typing import TabularData
 
@@ -31,9 +32,48 @@ def random_data_from_schema(
     max_session_length: Optional[int] = None,
     min_session_length: int = 5,
     device=None,
+    ragged=False,
+    seed=0,
 ) -> TabularData:
+    """Generates random tabular data based on a given schema.
+    The generated data can be used for testing
+    data preprocessing or model training pipelines.
+
+    Parameters
+    ----------
+    schema : Schema
+        The schema to be used for generating the random tabular data.
+    num_rows : int
+        The number of rows.
+    max_session_length : Optional[int]
+        The maximum session length.
+        If None, the session length will not be limited.
+        By default None
+    min_session_length : int
+        The minimum session length.
+        By default 5
+    device : torch.device
+        The device on which the synthetic data should be created.
+        If None, the synthetic data will be created on the CPU.
+        By default None
+    ragged : bool
+        If True, the sequence features will be represented with __values and __offsets.
+        By default False
+
+    Returns
+    -------
+    TabularData
+        A dictionary where each key is a feature name and each value is the generated
+        tensor.
+
+    """
     data: Dict[str, Any] = {}
 
+    random.seed(seed)
+    np.random.seed(seed)
+    if seed:
+        torch.manual_seed(seed)
+
     for i in range(num_rows):
         session_length = None
         if max_session_length:
@@ -87,13 +127,18 @@ def random_data_from_schema(
     for key, val in data.items():
         if isinstance(val, tuple):
             offsets = [0]
-            for length in val[1][:-1]:
-                offsets.append(offsets[-1] + length)
-            vals = (val[0], torch.tensor(offsets, device=device).unsqueeze(dim=1))
-            values, offsets, diff_offsets, num_rows = _pull_values_offsets(vals, device=device)
-            indices = _get_indices(offsets, diff_offsets, device=device)
-            seq_limit = max_session_length or val[1][0]
-            outputs[key] = _get_sparse_tensor(values, indices, num_rows, seq_limit)
+            for row_length in val[1]:
+                offsets.append(offsets[-1] + row_length)
+
+            if ragged:
+                outputs[f"{key}__values"] = val[0]
+                outputs[f"{key}__offsets"] = torch.tensor(offsets, device=device)
+            else:
+                vals = (val[0], torch.tensor(offsets[:-1], device=device).unsqueeze(dim=1))
+                values, offsets, diff_offsets, num_rows = _pull_values_offsets(vals, device=device)
+                indices = _get_indices(offsets, diff_offsets, device=device)
+                seq_limit = max_session_length or val[1][0]
+                outputs[key] = _get_sparse_tensor(values, indices, num_rows, seq_limit)
         else:
             outputs[key] = data[key]
 
