@@ -21,9 +21,12 @@ from typing import Dict, Optional, Union
 
 import numpy as np
 import torch
+from merlin.schema import Schema as CoreSchema
+from merlin.schema.io.proto_utils import has_field
+from merlin.schema.tags import Tags
 
 from merlin_standard_lib import Schema
-from merlin_standard_lib.utils.proto_utils import has_field
+from merlin_standard_lib.schema.schema import ColumnSchema
 
 from ...config.schema import SchemaMixin
 from ..typing import TabularData
@@ -137,22 +140,58 @@ def check_gpu(module):
         return False
 
 
+def _has_field(col_schema, field_name):
+    if isinstance(col_schema, ColumnSchema):
+        return has_field(col_schema, field_name)
+
+    return getattr(col_schema, field_name, None)
+
+
+def _get_size_from_shape(col_schema, batch_size) -> torch.Size:
+    shape = [batch_size]
+
+    if isinstance(col_schema, ColumnSchema):
+        if has_field(col_schema, "shape"):
+            shape += [d.size for d in col_schema.shape.dim]
+    elif col_schema.shape.dims is not None:
+        if len(col_schema.shape.dims) == 1 and col_schema.shape.dims[0].max is None:
+            return torch.Size(shape)
+        raise NotImplementedError("TODO: support shape.dims")
+
+    return torch.Size(shape)
+
+
 def get_output_sizes_from_schema(schema: Schema, batch_size=-1, max_sequence_length=None):
     sizes = {}
-    for feature in schema.feature:
+
+    features = schema if isinstance(schema, CoreSchema) else schema.feature
+
+    for feature in features:
         name = feature.name
+        # Pretrained embeddings (2-D or 3-D)
+        if Tags.EMBEDDING in feature.tags:
+            if len(feature.shape.dims) > 2:
+                sizes[name] = torch.Size(
+                    [
+                        batch_size,
+                        max_sequence_length if max_sequence_length else feature.value_count.max,
+                        feature.shape[-1].min,
+                    ]
+                )
+            else:
+                sizes[name] = torch.Size([batch_size, feature.shape[-1].min])
+
         # Sequential or multi-hot feature
-        if has_field(feature, "value_count"):
+        elif _has_field(feature, "value_count"):
             sizes[name] = torch.Size(
                 [
                     batch_size,
                     max_sequence_length if max_sequence_length else feature.value_count.max,
                 ]
             )
-        elif has_field(feature, "shape"):
-            sizes[name] = torch.Size([batch_size] + [d.size for d in feature.shape.dim])
+
         else:
-            sizes[name] = torch.Size([batch_size])
+            sizes[name] = _get_size_from_shape(feature, batch_size)
 
     return sizes
 
@@ -165,8 +204,8 @@ def calculate_batch_size_from_input_size(input_size):
 
 
 def check_inputs(ks, scores, labels):
-    if len(ks.shape) > 1:
-        raise ValueError("ks should be a 1-dimensional tensor")
+    if not (isinstance(ks, (list, tuple)) and len(ks) >= 1):
+        raise ValueError("ks should be a list or tuple with at least one element")
 
     if len(scores.shape) != 2:
         raise ValueError("scores must be a 2-dimensional tensor")
@@ -178,9 +217,9 @@ def check_inputs(ks, scores, labels):
         raise ValueError("scores and labels must be the same shape")
 
     return (
-        ks.to(dtype=torch.int32, device=scores.device),
+        ks,
         scores,
-        labels,  # .to(dtype=torch.float32, device=scores.device),
+        labels,
     )
 
 
@@ -242,7 +281,8 @@ def nested_concat(tensors, new_tensors, padding_index=-100):
 
 
 def torch_pad_and_concatenate(tensor1, tensor2, padding_index=-100):
-    """Concatenates `tensor1` and `tensor2` on first axis, applying padding on the second if necessary.
+    """Concatenates `tensor1` and `tensor2` on first axis, applying padding on the second as needed
+
     #TODO this method was copied from the latest version of HF transformers library to support
     dict outputs. So we should remove it when T4Rec is updated to use the latest version
     """
